/********************************************************************
Beschreibung:
Zwei Pollin-Evaluationboards mit jeweils einem RFM12 um miteineander 
zu kommunizieren. Auf den Boards wurden ATmega8 mit 8 MHz internen 
Takt eingesetzt.

Dieser Code ist auf beiden Board drauf.

Pinbelegung
-----------------------
ATMEGA8           RFM12
-----------------------
SCK               SCK
MISO              SDO
MOSI              SDI
SS                nSEL
PD2 (INT0)        nIRQ
PD5               LED1
PD6               LED2
VDD               VDD
PD7               FSK
PD3 (INT1)        FFIT
PB0               VDI
**********************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <compat/deprecated.h>

#define F_CPU 8000000UL
#define BAUD 19200UL

#define LED1_ON		PORTD |= (1 << PD5)
#define LED2_ON		PORTD |= (1 << PD6)
#define LED1_OFF	PORTD &= ~(1 << PD5)
#define LED2_OFF	PORTD &= ~(1 << PD6)
#define LED1_TOG	PORTD ^= (1 << PD5)
#define LED2_TOG	PORTD ^= (1 << PD6)

#define Taster3		PIND  &  (1<<PIND4)

// Funktionsprototypen
void RFM12_Write_Data(unsigned short);			
unsigned short RFM12_Write(unsigned short);
unsigned char RFM12_Read(void);			
void SPI_init(void);						
void RFM12_init(void);						
void USART_init(void);
unsigned char USART_get_byte(void);
void USART_send_byte(unsigned char);
void USART_send_int(char*);

unsigned char PacketLaenge, Empfangsbyte;				// Globale Variablen

ISR(INT1_vect)											// FIFO-Interrupt (PIN: FFIT)
{
	PacketLaenge=RFM12_Read();							// Erstes Nutzerbyte ist die Packetlnge
	for(char i=1;i<=PacketLaenge;i++)					// Restlichen Nutzerbytes ...
	{
		Empfangsbyte=RFM12_Read();						// ... auslesen				
		USART_send_byte(Empfangsbyte);					// Nutzerbyte ber UART an den PC schicken
		if(Empfangsbyte=='H')LED2_ON;					// LED2 an wenn Zeichen 'H' empfangen wird
		if(Empfangsbyte=='W')LED2_OFF;					// LED2 bei Zeichen 'W' wieder lschen
	}
	RFM12_Write(0xCA81);								// FIFO initialisieren
	RFM12_Write(0xCA83);								// FIFO deaktivieren 	
}

int main(void)
{
	unsigned char i=0;									// Zhlervariable	

	DDRD |= (1 << PD5) | (1 << PD6);					// PD5 (Led1), PD6 (Led2) als Ausgang		
	DDRD  &= ~(1<<PIND4);								// PD4 (Taster3) als Eingang

	SPI_init();											// SPI initialisieren
	RFM12_init();										// RFM12 initialisieren
	USART_init();										// USART initialisieren
	
	sei();												// Interrupts aktivieren
	while(1)											// Arbeitsschleife
	{
		if(Taster3)										// Wenn Taster3 gedrckt wird...
		{
			_delay_ms(100);								// ... 100ms warten (Tastenentprellung)
			if(Taster3)									// Wenn Taster3 noch gedrckt ...
			{
				i++;									// Zhlervariable incrementieren

			  	RFM12_Write(0x0000);					// Status Register lesen (Interrupts werden gelscht)
			  	RFM12_Write(0x8271);					// Sender aktivieren

				if(i==1)
				{
					RFM12_Write_Data(0xB8AA);			// Prambel senden			
					RFM12_Write_Data(0xB8AA);			// Prambel senden						
					RFM12_Write_Data(0xB8AA);			// Prambel senden						
					RFM12_Write_Data(0xB82D);			// Syncronmuster senden			
					RFM12_Write_Data(0xB8D4);			// Syncronmuster senden				
					RFM12_Write_Data(0xB800 | 6 );		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'H');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'e');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'l');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'l');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'o');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | ' '); 	// Nutzerbyte senden	
					RFM12_Write_Data(0xB8AA);			// Dummybyte
				}
				

				if(i==2)
				{
					RFM12_Write_Data(0xB8AA);			// Prambel senden			
					RFM12_Write_Data(0xB8AA);			// Prambel senden						
					RFM12_Write_Data(0xB8AA);			// Prambel senden						
					RFM12_Write_Data(0xB82D);			// Syncronmuster senden			
					RFM12_Write_Data(0xB8D4);			// Syncronmuster senden				
					RFM12_Write_Data(0xB800 | 6 );		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'W');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'o');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'r');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'l');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | 'd');		// Nutzerbyte senden							
					RFM12_Write_Data(0xB800 | ' '); 	// Nutzerbyte senden	
					RFM12_Write_Data(0xB8AA);			// Dummybyte
				}					
				if(i==2)i=0;
			}
		}				
	}
}

void RFM12_Write_Data(unsigned short wert)
{
	while(PIND & (1<<PIND2));							// Warten bis Modul bereit
	RFM12_Write(wert);									// Sendeword bertragen
}

unsigned short RFM12_Write(unsigned short wert)
{
	unsigned short newHigh=0,newLow=0,newWord=0;		// Typendeklaration
	PORTB &= ~(1<<PB2);									// SS am Slave Low --> Beginn der bertragung
	SPDR = wert >> 8;									// High-Byte bertragen
	while (!(SPSR & (1<<SPIF)));						// Warten bis bertragung abgeschlossen
	newHigh = SPDR;										// Empfangenes High-Byte lesen
	SPDR = wert;										// Low-Byte bertragen
	while (!(SPSR & (1<<SPIF)));						// Warten bis bertragung abgeschlossen
	newLow = SPDR;										// Empfangenes Low-Byte lesen
	PORTB |= (1<<PB2);									// SS am Slave High --> Ende der bertragung
	newWord = (newHigh<<=8) | newLow;					// High-Byte und Low-Byte zusammenfhren	
	return newWord;										// Rckgabe des empfangenen Words
}

unsigned char RFM12_Read(void)
{
	unsigned short FIFO_data;							// Typendeklaration
	RFM12_Write(0x0000);								// Status Register lesen (Interrupts werden gelscht)
	while(PIND & (1<<PIND2));							// Warten bis Modul bereit
  	FIFO_data=RFM12_Write(0xB000);						// FIFO-Daten einlesen (16Bit)
	return(FIFO_data&0x00FF);							// LOW-Byte der FIFO-Daten zurckgeben (Datenbyte)
}

void SPI_init (void) 
{
	unsigned char status;
	DDRB = (1<<PB2) | (1<<PB3) | (1<<PB5);				// setze SCK,MOSI,PB2 (SS) als Ausgang
	DDRB &= ~(1<<PB4);									// setze MISO als Eingang
	PORTB = (1<<PB5) | (1<<PB2);						// SCK und PB2 high (ist mit SS am Slave verbunden)
	SPCR = (1<<SPE) | (1<<MSTR);						// Aktivierung des SPI, Master, Taktrate fck/4
	status = SPSR;										// Status lschen
}

void RFM12_init(void)
{
  	RFM12_Write(0x80D8);								// TX-FIFO, RX-FIFO, Band = 433 MHz, Quarz = 12,5 pF 	
	RFM12_Write(0x82D1);								// Empfnger, Basis-Band, Synthesizer, Kein Takt-Ausgang
  	RFM12_Write(0xA140);								// Sendefrequenz = 430.8 MHz
	RFM12_Write(0xC623);								// 9600 Bit/s
	RFM12_Write(0x9480);								// VDI, Schnell, 200 KHz, 0 dB, -103 dB
  	RFM12_Write(0xC2AC);								// Auto-Lock, Digitalfilter, mittlere Schwelle
  	RFM12_Write(0xCA81);								// FIFO = 8 Bit, Synchronmuster, FIFO leeren/sperren = Empfnger, Reset-Schaltung aus
  	RFM12_Write(0xC4F7);								// Auto-Mode = VDI unabhngig, Range-Limit = -4/+3, Fine-Mode, Offset aktiv, AFC aktiv
  	RFM12_Write(0x9800);								// Modulation-Polarity = normal, Frequenzhub = 15 KHz, Ausgangsleistung = Maximal
	RFM12_Write(0xCA81);								// FIFO initialisieren
	RFM12_Write(0xCA83);								// FIFO aktivieren

	DDRD  |=  (1<<PD7);									// PD7 (nFFS) als Ausgang
 	PORTD |=  (1<<PD7);									// nFFS-Pin auf high ziehen

	
	DDRD  &= ~(1<<PIND2);				 				// PD2(INT0)	
	DDRD  &= ~(1<<PIND3);				 				// PD3(INT1) fr den FIFO-Interrupt

	MCUCR |= (1<<ISC10) | (1<<ISC11);					// INT1 steigende Flanke
	GICR  |= (1<<INT1);									// INT1 aktivieren
}

void USART_init(void)
{
	unsigned char x;
	UBRRL = (F_CPU / (16ul * BAUD))-1;						// Baudrate mit Takt und Baud
	UCSRB |= (1 << TXEN) | (1 << RXEN); 					// Sender und Empnger aktivieren
	UCSRC |= (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);	// async 8bit
	x = UDR;												// Empfnger leeren
	sbi(UCSRB,RXCIE );										// USART-Empfangs-Interrupt aktivieren
}

unsigned char USART_get_byte(void)
{
	while(!(UCSRA & (1 << RXC)));							// warten bis Empfang abgeschlossen
	return UDR;												// Byte abholen
}

void USART_send_byte(unsigned char x)
{
	while(!(UCSRA & (1 << UDRE)));							// warten bis Schnittstelle bereit
	UDR = x;												// Zu sendendes Byte bergeben
}

void USART_send_int(char *s)
{
    while (*s)												// so lange *s ungleich '\0' (String-Endezeichen)
    {   
        USART_send_byte(*s);								// Byte senden
        s++;												// nchstes Byte whlen
    }
	USART_send_byte('\r');									// Carriage Return senden
}

ISR(USART_RXC_vect)											// USART Empfangs-Interrupt
{
	unsigned char daten;
	daten = USART_get_byte();								// Zeichen aus Puffer holen
}

